-----------------------------------------------------
-- lua 파일이 여러개인 경우
-- 애드온명, 애드온내공유테이블이 ...으로 전달됨
-----------------------------------------------------
local addonName, L = ...;

--------------------------------------------------------------------------
-- 20191111
-- 아이템 툴팁이 화면밖으로 나가는 현상 처리
-- IsClampedToScreen() 값이 FALSE(화면밖이동허용) 이면, 다시 복구처리함.
-- 확인 1) 이 현상은 전투중 몹의 버프, 디버프 툴팁을 확인했을때 나타남.
--       ==> 툴팁을 띄우는 방법 확인 필요
-- 진득추측)
-- 이름표는 언제든 화면을 벗어날수 있다.(화면 돌림등.)
-- 하지만, 화면내 고정하는 옵션이 문제될수 있어서 제외한듯.(8.2.5쯤 부터)
-- 이름표뿐 아니라 툴팁이 화면을 벗어날수 있는 Widget이 많으므로..
--------------------------------------------------------------------------
function on_GameTooltip_Show(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    --DebugPrintTable(self);
    
    if( not self:IsClampedToScreen() ) then
        --print( self:GetName().."[|cff000000tooltip:IsClampedToScreen change to : "..tostring(self:IsClampedToScreen()).."|r]" );
        ------------------------------------------------------------------------------------------
        -- 20191129
        -- self:SetClampedToScreen(true); 
        -- --> Action[ClampedToScreen] failed because[Can't clamp restricted regions] 오류발생함.
        -- nameplate 특성상 고정되어 있지 않아서 인듯.
        -- lua의 try catch문, pcall 이용하여 pass
        ------------------------------------------------------------------------------------------
        -- self:SetClampedToScreen(true); 
        local status, retval = pcall(self.SetClampedToScreen, self, true);
        --print(status);
        --print(retval);
    end
end
GameTooltip:HookScript("OnShow",on_GameTooltip_Show);

-- 20190927
-- Action 버튼, 케릭터 등 화면하단에 고정된 툴팁 위치 이동
-- (참조 asTooltip)
function on_GameTooltip_SetDefaultAnchor(tooltip, parent,...)
    -- Return if no tooltip or parent
    if (not tooltip or not parent) then
        return;
    end

    -- 20190927
    -- 툴팁이동설정시에만 기능동작하도록 수정함.
    if not LJDSpellInterruptDB["ReLocateGameToolTip"] then
        return;
    end

    local AFFL_X = 385;                  --툴팁 X 위치

    tooltip:SetOwner(parent, "ANCHOR_NONE");
    tooltip:ClearAllPoints();
    tooltip:SetPoint("BOTTOM", "UIParent", "BOTTOM", AFFL_X, getTooltipBottomPosY());
    tooltip:SetScale(0.9);
end
hooksecurefunc("GameTooltip_SetDefaultAnchor", on_GameTooltip_SetDefaultAnchor);

--------------------------------------------------------------------------
-- 20191114
-- 버프/디버프 아이콘 툴팁에도 시전자 표시
--------------------------------------------------------------------------
function on_GameTooltip_SetUnitAura(self, ...)
    local casterId = select(7, UnitAura(...));
    --DebugPrint(casterId);
    local unitId, auraIndex, filter = ...;

    if casterId then

        if not UnitIsUnit(casterId, unitId) then
            local className, classFilename, classID = UnitClass(casterId);
            local rPerc, gPerc, bPerc, argbHex = GetClassColor(classFilename);
            local casterName = UnitName(casterId);

            self:AddLine(" ");
            self:AddDoubleLine(L["시전"]..":", casterName, _, _, _, rPerc, gPerc, bPerc);
            self:Show();
        end
    end
end
hooksecurefunc(GameTooltip, "SetUnitAura", on_GameTooltip_SetUnitAura);

--------------------------------------------------------------------------
-- 20191111
-- 아이템, 스킬, 퀘스트 참조 툴팁(채팅창의 링크 클릭해서 열리는)이 
-- 스킬창을 가리는 것을 해결하기 위해 위치 조정.
--------------------------------------------------------------------------
function on_ItemRefTooltip_Show(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    --DebugPrintTable(self);

    -- 툴팁이동설정시에만 기능동작하도록 수정함.
    if not LJDSpellInterruptDB["ReLocateGameToolTip"] then
        return
    end

    --self:SetOwner(self:GetParent(), "ANCHOR_NONE");   -- 특이점.. 이 라인 주석해제시 툴팁이 나오지 않음.
    self:ClearAllPoints();
    self:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, getTooltipBottomPosY());
end
ItemRefTooltip:HookScript("OnShow", on_ItemRefTooltip_Show);

--------------------------------------------------------------------------
-- 20191111
-- 아이템 툴팁의 전체 보유량 표시
--------------------------------------------------------------------------
function on_GameTooltip_SetItem(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    --DebugPrintTable(self);
    local itemName, itemLink = self:GetItem();
    local itemCount = GetItemCount(itemLink, true);   -- 은행포함.(2번째 인자)
    
    --print(itemName..":"..itemCount);
    if itemName == '' then
        return;
    end
    
    self:AddLine("OnTooltipSetItem");
    self:AddDoubleLine(L["보유"]..":", itemCount, _, _, _, 1, 1, 1);
    self:Show();
            
end
--GameTooltip:HookScript("OnTooltipSetItem", on_GameTooltip_SetItem);

-- 재료템
function on_GameTooltip_SetRecipeReagentItem(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    --DebugPrintTable(self);
    local recipe, reagent = ...;
    --print(C_TradeSkillUI.GetRecipeReagentItemLink(recipe, reagent));
    
    local itemLink = C_TradeSkillUI.GetRecipeReagentItemLink(recipe, reagent);
    local itemCount = GetItemCount(itemLink, true);   -- 은행포함.(2번째 인자)
    --print(itemLink..":"..itemCount);

    self:AddLine("SetRecipeReagentItem");
    self:AddDoubleLine(L["보유"]..":", itemCount, _, _, _, 1, 1, 1);
    self:Show();

end
--hooksecurefunc(GameTooltip, 'SetRecipeReagentItem', on_GameTooltip_SetRecipeReagentItem)

-- 결과템
function on_GameTooltip_SetRecipeResultItem(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    --DebugPrintTable(self);
    local recipe = ...;
    --print(C_TradeSkillUI.GetRecipeItemLink(recipe));
    
    local itemLink = C_TradeSkillUI.GetRecipeItemLink(recipe);
    local itemCount = GetItemCount(itemLink, true);   -- 은행포함.(2번째 인자)
    --print(itemLink..":"..itemCount);
    
    self:AddLine("SetRecipeResultItem");
    self:AddDoubleLine(L["보유"]..":", itemCount, _, _, _, 1, 1, 1);
    self:Show();
end
--hooksecurefunc(GameTooltip, 'SetRecipeResultItem', on_GameTooltip_SetRecipeResultItem);

---------------------------------------------------------------------------
-- 사용한 스킬과 적용된 오라의 ID는 다른다.
-- ex) 황소돌진의 스킬ID는 255654, 이 스킬로 인해 생긴 디버프 ID : 255723
---------------------------------------------------------------------------

-- Aura Id 표현(디버깅용)
function on_GameTooltip_ShowAuraSpellId(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    
    local unitId = ll[1];
    local auraIx = ll[2];
    local filter = ll[3];

    -- [스킬ID 보기] 설정에 따라 표현
    if( not LJDSpellInterruptDB["ShowSkillId"] ) then
        return;
    end
    if( not unitId ) then return end;
    
    local auraInfo = {UnitAura(unitId, auraIx, filter)};
    local spellNm = auraInfo[1];
    local spellID = auraInfo[10];
    
    --DebugPrint(unitId..":"..spellNm .. ":" .. spellID);
    
    --self:AddDoubleLine(L["ID"]..":", spellID, _, _, _, _, _, _);
    --self:Show(); -- ReDraw
    
    local textRight1 = _G[self:GetName().."TextRight1"];
    if( textRight1 and spellID ) then
        textRight1:SetText("|c".."ffff00ff"..spellID.."|r");   -- 보라색
        textRight1:Show();  -- 기본숨김처리 되어있음.
        self:Show(); -- ReDraw
    end
end
hooksecurefunc(GameTooltip, "SetUnitAura"  , on_GameTooltip_ShowAuraSpellId);
hooksecurefunc(GameTooltip, "SetUnitBuff"  , on_GameTooltip_ShowAuraSpellId);
hooksecurefunc(GameTooltip, "SetUnitDebuff", on_GameTooltip_ShowAuraSpellId);


-- Spell Id 표현(디버깅용)
function on_GameTooltip_ShowSpellId(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    
    local spellName, spellID = self:GetSpell();

    -- [스킬ID 보기] 설정에 따라 표현
    if( not LJDSpellInterruptDB["ShowSkillId"] ) then
        return;
    end
    if( not spellID ) then return end;
    
    --self:AddDoubleLine(L["ID"]..":", spellID, _, _, _, _, _, _);
    --self:Show(); -- ReDraw

    local textRight1 = _G[self:GetName().."TextRight1"];
    if( textRight1 and spellID ) then
        textRight1:SetText("|c".."ffff00ff"..spellID.."|r");   -- 보라색
        textRight1:Show();  -- 기본숨김처리 되어있음.
        self:Show(); -- ReDraw
    end

end
GameTooltip:HookScript("OnShow"  ,on_GameTooltip_ShowSpellId);
GameTooltip:HookScript("OnUpdate",on_GameTooltip_ShowSpellId);
